@@ -54,6 +54,19 @@ class KubeclientTest < MiniTest::Test
     assert_equal(watch_client.send(:build_client_options)[:proxy][:proxy_port], proxy_uri.port)
   end
 
+  def test_pass_max_redirects
+    max_redirects = 0
+    client = Kubeclient::Client.new('http://localhost:8080/api/', http_max_redirects: max_redirects)
+    rest_client = client.rest_client
+    assert_equal(max_redirects, rest_client.options[:max_redirects])
+
+    stub_request(:get, 'http://localhost:8080/api')
+      .to_return(status: 302, headers: { location: 'http://localhost:1234/api' })
+
+    exception = assert_raises(Kubeclient::HttpError) { client.api }
+    assert_equal(302, exception.error_code)
+  end
+
   def test_exception
     stub_core_api_list
     stub_request(:post, %r{/services})
@@ -283,6 +296,22 @@ class KubeclientTest < MiniTest::Test
     )
   end
 
+  def test_entities_with_resource_version
+    version = '329'
+
+    stub_core_api_list
+    stub_get_services
+
+    services = client.get_services(resource_version: version)
+
+    assert_instance_of(Kubeclient::Common::EntityList, services)
+    assert_requested(
+      :get,
+      "http://localhost:8080/api/v1/services?resourceVersion=#{version}",
+      times: 1
+    )
+  end
+
   def test_entities_with_field_selector
     selector = 'involvedObject.name=redis-master'
 
